package linucb

import (
	"bytes"
	"compress/zlib"
	"encoding/binary"
	"fmt"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/log"
	"io"
	"math"
	"unsafe"
)

func Decompress(input []byte) []byte {
	buf := bytes.NewBuffer(input)
	reader, _ := zlib.NewReader(buf)
	defer reader.Close()

	var out bytes.Buffer
	_, err := io.Copy(&out, reader)
	if err != nil {
		log.Error(fmt.Sprintf("decompress error=%v", err))
	}
	return out.Bytes()
}

func ToFloat32(bytes []byte) float32 {
	bits := binary.BigEndian.Uint32(bytes)
	return math.Float32frombits(bits)
}

func ToByteArray(num int32) []byte {
	size := int(unsafe.Sizeof(num))
	arr := make([]byte, size)
	for i := 0; i < size; i++ {
		byt := *(*uint8)(unsafe.Pointer(uintptr(unsafe.Pointer(&num)) + uintptr(i)))
		arr[i] = byt
	}
	return arr
}

func BytesToFloat32(bytes []byte) []float32 {
	length := len(bytes) / 4
	result := make([]float32, 0, length)
	for i := 0; i < length; i++ {
		result = append(result, ToFloat32(bytes[i*4:(i+1)*4]))
	}
	return result
}

func ToDenseArray(values []float32) []float64 {
	result := make([]float64, 0, len(values)*2)
	for _, v := range values {
		if math.IsNaN(float64(v)) {
			length := getNumberFromFloatNaNWithNumber(v)
			for i := uint32(0); i <= length; i++ {
				result = append(result, 0)
			}
		} else {
			result = append(result, float64(v))
		}
	}
	return result
}

func getNumberFromFloatNaNWithNumber(value float32) uint32 {
	i := math.Float32bits(value)
	i = i & 0x0000FFFF
	return i
}

func Decode(values []int32) []byte {
	length := values[0]
	capacity := length
	if length%4 > 0 {
		capacity++
	}
	result := make([]byte, 0, capacity)
	for _, v := range values[1:] {
		result = append(result, ToByteArray(v)...)
	}
	if capacity == length {
		return result
	} else {
		return result[:length]
	}
}

func Parse(values []int32) []float64 {
	if len(values) == 0 {
		return nil
	}
	byteArray := Decode(values)
	decompressed := Decompress(byteArray)
	floatValues := BytesToFloat32(decompressed)
	return ToDenseArray(floatValues)
}
