package module

import (
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/context"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/recconf"
)

const (
	Feature_Store_User = "user"
	Feature_Store_Item = "item"
)

// FeatureDao is interface, it define FeatureFetch to featch feature from implement
type FeatureDao interface {
	FeatureFetch(user *User, items []*Item, context *context.RecommendContext)
}

// NewFeatureDao create FeatureDao from config
// config.AdapterType is decide the implement
func NewFeatureDao(config recconf.FeatureDaoConfig) FeatureDao {
	if config.AdapterType == recconf.DaoConf_Adapter_Redis {
		return NewFeatureRedisDao(config)
	} else if config.AdapterType == recconf.DaoConf_Adapter_Hologres {
		return NewFeatureHologresDao(config)
	} else if config.AdapterType == recconf.DaoConf_Adapter_TableStore {
		return NewFeatureTablestoreDao(config)
	}

	panic("not found FeatureDao implement")
}

type FeatureBaseDao struct {
	featureKey   string // use the value of key to featch data
	featureStore string // user or item
}

func NewFeatureBaseDao(config *recconf.FeatureDaoConfig) *FeatureBaseDao {
	dao := FeatureBaseDao{
		featureKey:   config.FeatureKey,
		featureStore: config.FeatureStore,
	}

	return &dao
}
