package service

import (
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/context"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/datasource"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/module"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/recconf"
)

type CallBackService struct {
	featureService *FeatureService
	User           *module.User
	Items          []*module.Item
}

func NewCallBackService() *CallBackService {
	services := CallBackService{
		featureService: DefaultFeatureService(),
	}
	return &services
}

func (r *CallBackService) LoadLog(context *context.RecommendContext) {
	r.featureService.LoadFeatures(r.User, r.Items, context)
}

func (r *CallBackService) RecordLog(context *context.RecommendContext, msg string) error {
	scene_name := context.GetParameter("scene").(string)
	if callBackConfig, ok := context.Config.CallBackConfs[scene_name]; ok {
		data_source := callBackConfig.DataSource
		if data_source.Type == recconf.DataSource_Type_Kafka {
			return r.RecordToKafka(data_source.Name, msg)
		}
	}

	return nil
}

func (r *CallBackService) RecordToKafka(kafka_name string, msg string) error {
	p, error := datasource.GetKafkaProducer(kafka_name)
	if error != nil {
		return error
	}
	p.SendMessage(msg)
	return nil
}
