package sort

import (
	"fmt"

	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/context"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/log"
	"gitlab.alibaba-inc.com/pai_biz_arch/pairec/recconf"
)

var sortMapping = make(map[string]ISort)
var sortService *SortService

// var sortMapping map[string]ISort

func init() {
	sortService = &SortService{}
	sortService.SortStrategies = make(map[string][]ISort, 0)
}

type SortData struct {
	Data    interface{}
	Context *context.RecommendContext
}
type ISort interface {
	Sort(sortData *SortData) error
}

type SortService struct {
	SortStrategies map[string][]ISort
}

func (ss *SortService) AddSort(scene string, s ISort) {
	sorts, ok := ss.SortStrategies[scene]
	if ok {
		for _, sort := range sorts {
			if sort == s {
				return
			}
		}

		sorts = append(sorts, s)
		ss.SortStrategies[scene] = sorts
	} else {
		sorts := []ISort{s}
		ss.SortStrategies[scene] = sorts
	}
}
func (ss *SortService) Sort(data *SortData, tag string) {

	var scene string
	s := data.Context.GetParameter("scene")
	if _, ok := s.(string); ok {
		scene = s.(string)
	}
	scene = scene + tag
	sorts, ok := ss.SortStrategies[scene]
	if !ok {
		sorts, ok = ss.SortStrategies["default"]
	}
	if sorts == nil {
		if tag == "_PreSort" {
			sorts = make([]ISort, 1)
			sorts[0] = NewItemRankScoreSort()
		} else {
			log.Error(fmt.Sprintf("requestId=%s\tmsg=sort not find\tscene=%s", data.Context.RecommendId, scene))
			return
		}
	}
	// var err error
	for _, s := range sorts {
		s.Sort(data)
	}
}

func Load(config *recconf.RecommendConfig) {
	for scene, names := range config.SortNames {
		for _, name := range names {
			if sort, ok := sortMapping[name]; ok {
				sortService.AddSort(scene, sort)
			}
		}
	}
}

func Sort(sortData *SortData, tag string) {
	sortService.Sort(sortData, tag)
}
func RegisterSort(name string, s ISort) {
	if s == nil {
		panic("Sort is nil, name:" + name)
	}

	sortMapping[name] = s
}
